/*
	Logomatic V2 Firmware
	SparkFun Electronics 2008
	
	Basic, event based text logging
	
	When something happens, write some text to a file
 
 */

/* Header Files */
//======================================
#include <stdio.h>
#include <string.h>
#include "LPC21xx.h"

//UART0 Debugging
#include "serial.h"
#include "rprintf.h"

//Needed for main function calls
#include "string_printf.h"
#include "fat16.h"
#include "rootdir.h"
#include "sd_raw.h"

/* Global Variables */
//======================================
#define ON	0
#define OFF	1

#define STAT1	9
#define STAT0	10

struct fat16_file_struct* handle;

/* Function Declarations */
//======================================
void UNDEF_Routine(void) __attribute__ ((interrupt("UNDEF")));

void boot_up(void);
void log_string(char *buf);

void card_init(void);

char look_up_pin(char labeled_pin);
void set_pin_mode(char pin_number, char *buf);
char read_digital_pin(char pin_number);
void write_digital_pin(char pin_number, char pin_state);
void flash_it(int);

void error_mode(char error_number);
void delay_ms(int count);
//======================================

int main (void)
{
	int rtc_seconds = 0;
	int bubble_number = 0;
	
	boot_up(); //Init LPC, IO pins, and FAT
	
	log_string("Log started\n\r");

	set_pin_mode(1, "DI");
	
	while(1)
	{
		if(read_digital_pin(1) == 1) //See if photo gate is activated
		{
			bubble_number++;
			
			char str_data[128];
			string_printf(str_data, "Bubble,%05d,%05d\n\r", bubble_number, rtc_seconds);
			log_string(str_data);
		
			write_digital_pin(STAT0, ON); //Turn on LED
			delay_ms(50);
			write_digital_pin(STAT0, OFF); //Turn off LED
			delay_ms(250);
		}
		
		if(T1TC > 1000)
		{
			T1TC -= 1000;
			rtc_seconds++;
		
			write_digital_pin(STAT1, ON); //Turn on LED
			
			for(int j = 0 ; j < 5 ; j++)
			{
				delay_ms(1);
				if(read_digital_pin(1) == 1) break;
			}
			write_digital_pin(STAT1, OFF); //Turn off LED
		}
	}

    return 0;
}

void log_string(char *buf)
{
	int stringSize = strlen(buf);
	fat16_write_file(handle, (unsigned char*)buf, stringSize);
	sd_raw_sync();
}

//Basic file init and stuff
void boot_up(void)
{
	set_pin_mode(STAT0, "DO"); //Set status LEDs as outputs
	set_pin_mode(STAT1, "DO");

	rprintf_devopen(putc_serial0); //Open up serial port 0 for debugging

	card_init(); //Init SD card interface
	
	//Creat the main log file
	char name[32];
	int count = 0;
	string_printf(name,"LOG%02d.txt",count);
	while(root_file_exists(name))
	{
		count++;
		if(count == 250) 
		{
			rprintf("Too Many Logs!\n\r");
			error_mode(4);
		}
		string_printf(name,"LOG%02d.txt",count);
	}
	handle = root_open_new(name);

	sd_raw_sync();

	//Setup TIMER1 to run
	T1TCR = 0; //Reset Timer1
	T1PR = 60000; //Set prescalar to 60,000 - every tick is 1ms
	T1TCR = 1; //Start Timer1

	flash_it(50); //Flash Status Lights
}

//Turn on/off the two status LEDs simultaneously
void flash_it(int length_of_flash)
{
	for(int i = 0; i < 5; i++)
	{
		write_digital_pin(STAT0, OFF); //Turn off LED
		write_digital_pin(STAT1, ON); //Turn on LED
		delay_ms(length_of_flash);
		
		write_digital_pin(STAT0, ON); //Turn off LED
		write_digital_pin(STAT1, OFF); //Turn off LED
		delay_ms(length_of_flash);
	}
}

//Sets the state of a digital output
void write_digital_pin(char pin_number, char pin_state)
{
	pin_number = look_up_pin(pin_number); //Translate labeled pins to pins on LPC

	if(pin_state == 1) //Turn pin on
		IOSET0 |= (1<<pin_number);
	else //Turn pin off
		IOCLR0 |= (1<<pin_number);
}

//Reads the stat of a digital input
char read_digital_pin(char pin_number)
{
	pin_number = look_up_pin(pin_number); //Translate labeled pins to pins on LPC
	
	if(IOPIN0 & (1<<pin_number)) return(1); //If pin is high, return a 1
	return(0); //else return 0
}

//Does a lookup from the Logomatic labeled pins D1 through D8
//Sets the function of the pin as digital or analog
//Sets the pin as either an input or an output
//None of the P0 pins have internal pull ups? Page 70
void set_pin_mode(char pin_number, char *buf)
{
	pin_number = look_up_pin(pin_number); //Translate labeled pins to pins on LPC

	if (buf[0] == 'D') //Digital IO
	{
		if(pin_number > 15) //Then we are in PINSEL1 register
		{
			pin_number -= 16;
			PINSEL1 = (PINSEL1 & ~(3<<(pin_number*2))); //Clear the bits to force GPIO function
		}
		else
			PINSEL0 = (PINSEL0 & ~(3<<(pin_number*2))); //Clear the bits to force GPIO function
	}
	
	//Aanlog functions don't fully map correctly to the PINSEL registers
	/*if (buf[0] == 'A') //Analog IO
	{
		if(actual_pin > 15) //Then we are in PINSEL1 register
		{
			actual_pin -= 16;
			PINSEL1 = (PINSEL1 & ~(3<<(actual_pin*2))) | (1<<(actual_pin*2)); //Clear the bits then set 1 bit
		}
		else
			PINSEL0 = (PINSEL0 & ~(3<<(actual_pin*2))) | (1<<(actual_pin*2)); //Clear the bits then set 1 bit
	}*/
	
	//1 = output 0 = input
	if (buf[1] == 'I') //Input
		IODIR0 &= ~(1<<pin_number); //Set pin as input
	if (buf[1] == 'O') //Output
		IODIR0 |= (1<<pin_number); //Set pin as output
}

//Given a pin number like 3, this translates it to the actual pin on the LPC
char look_up_pin(char labeled_pin)
{
	#define PIN1 	30
	#define PIN2	29
	#define PIN3	28
	#define PIN4	25
	#define PIN5	22
	#define PIN6	21
	#define PIN7	10
	#define PIN8	12
	#define PIN9	11
	#define PIN10	2

	char actual_pin;
	switch(labeled_pin)
	{
		case 1: actual_pin = PIN1; break;
		case 2: actual_pin = PIN2; break;
		case 3: actual_pin = PIN3; break;
		case 4: actual_pin = PIN4; break;
		case 5: actual_pin = PIN5; break;
		case 6: actual_pin = PIN6; break;
		case 7: actual_pin = PIN7; break;
		case 8: actual_pin = PIN8; break;
		case 9: actual_pin = PIN9; break;
		case 10: actual_pin = PIN10; break;
		default: actual_pin = 0;
	}
	return(actual_pin);
}

void card_init(void)
{
	if(!sd_raw_init()) //Attempt to init SD raw mode
	{
		rprintf("SD Init Error\n\r");
		error_mode(2);
	}

	if(openroot()) //Attempt to open root directory
	{ 
		rprintf("SD OpenRoot Error\n\r");
		error_mode(3);
	}
}

//Blinks stat LED in an error number
void error_mode(char error_number)
{
	error_number++;
	
	while(1)
	{
		for(char i = 0 ; i < error_number ; i++)
		{
			write_digital_pin(STAT0, ON);
			delay_ms(250);
			write_digital_pin(STAT0, OFF);
			delay_ms(250);
		}

		delay_ms(2000);
	}
}

//Not really true, but close to a delay of a milisecond
void delay_ms(int count)
{
	int i;
	count *= 10000;
	for(i = 0; i < count; i++)
		asm volatile ("nop");
}
